<?php
/*
Plugin Name: WordPress HTTPS
Plugin URI: http://mvied.com/projects/wordpress-https/
Description: This plugin ensures that <em>all</em> elements on a page are accessed through HTTPS if the page is being accessed via HTTPS.
Author: Mike Ems
Version: 0.3
Author URI: http://mvied.com/
*/

add_action('admin_menu', 'wordpress_https_menu');

function wordpress_https_menu() {
 add_options_page('WordPress HTTPS Settings', 'WordPress HTTPS', 'manage_options', 'wordpress-https', 'wordpress_https_settings');
}

function wordpress_https_settings() {
 if (!current_user_can('manage_options')) {
  wp_die( __('You do not have sufficient permissions to access this page.') );
 }

 if ( $_SERVER['REQUEST_METHOD'] === 'POST' ) {
  $errors = array();
  if ( $_POST['wordpress-https_externalurls'] == '1' ) {
   if ( file_get_contents('http://www.google.com') == false ) {
    $errors[] = 'Your current server configuration does not support the \'External HTTPS Elements\' option. This functionality relies on the <a href="http://php.net/file_get_contents" target="_blank">file_get_contents</a> function in PHP and has failed a test call to \'http://www.google.com/\'.';
    update_option('wordpress-https_externalurls', 0);
   } else {
    update_option('wordpress-https_externalurls', 1);
   }
  } else {
   update_option('wordpress-https_externalurls', 0);
  }
 }
?>
<div class="wrap">
 <div id="icon-options-general" class="icon32"><br /></div>
 <h2>WordPress HTTPS Settings</h2>
<?php
 if ( $_SERVER['REQUEST_METHOD'] === 'POST' ) {
  if ( sizeof( $errors ) > 0 ) {
   echo " <ul class=\"errors\">\n";
   foreach ( $errors as $error ) {
    echo "  <li class=\"error\"><p>".$error."</p></li>\n";
   }
   echo " </ul>\n";
  } else {
   echo "  <div class=\"updated fade\" id=\"message\"><p><strong>Settings saved.</strong></p></div>\n";
  }
 }
?>

 <form name="form" action="options-general.php?page=wordpress-https" method="post">
  <?php settings_fields('wordpress-https'); ?>
  <table class="form-table">
   <tr valign="top">
    <th scope="row">External HTTPS Elements</th>
    <td>
     <fieldset>
      <legend class="screen-reader-text"><span>Attempt to automatically convert external elements to HTTPS.</span></legend>
      <label for="wordpress-https_externalurls"><input name="wordpress-https_externalurls" type="checkbox" id="wordpress-https_externalurls" value="1"<?php echo ((get_option('wordpress-https_externalurls')) ? ' checked="checked"' : ''); ?> /> Attempt to automatically convert external elements to HTTPS.<br /><em>Warning: Depending on the amount of external elements, this could affect the load times of your pages.</em></label>
     </fieldset>
    </td>
   </tr>
  </table>
  <p class="submit"><input type="submit" name="Submit" value="Save Changes" class="button-primary" /></p>
  <p>If you found my plugin useful, please <a href="https://www.paypal.com/cgi-bin/webscr?cmd=_s-xclick&hosted_button_id=6ZL95VTJ388HG" target="_blank">donate</a>. If you need technical assistance, want to provide feedback, or just want to say thanks, <a href="http://wordpress.org/tags/wordpress-https#postform" target="_blank">drop me a line</a>.</p>
 </form>
</div>
<?php
}

function wordpress_https_process($buffer) {
 if ( !empty($_SERVER['HTTPS']) || $_SERVER['SERVER_PORT'] == 443 ) {
  $httpsUrl = str_replace('http://', 'https://', get_bloginfo('siteurl'));
  $httpUrl  = str_replace('https://', 'http://', get_bloginfo('siteurl'));

  preg_match_all('/\<(script|link|img)(.*)http:\/\/[-\w\.]+[^\"\']*/im',$buffer,$matches);
  foreach ($matches[0] as $match) {
   if ( ( strpos($match, 'link') !== false && strpos($match, 'stylesheet') !== false ) || strpos($match, 'script') !== false || strpos($match, 'img') !== false ) {
    if ( strpos($match,$httpUrl) !== false ) {
     $buffer = str_replace($match, str_replace('http', 'https', $match), $buffer);
    } elseif ( get_option('wordpress-https_externalurls') == '1' ) {
     preg_match_all('/http:\/\/[-\w\.]+[^\"\']*/im', $match, $urlMatches);
     foreach ( $urlMatches[0] as $urlMatch ) {
      if (@file_get_contents(str_replace('http', 'https', $urlMatch))) {
       $buffer = str_replace($urlMatch, str_replace('http', 'https', $urlMatch), $buffer);
      }
     }
    }
   }
  }
 }
 return $buffer;
}

function wordpress_https_buffer_start() { ob_start('wordpress_https_process'); }
function wordpress_https_buffer_end() { ob_end_flush(); }

add_action('admin_init', 'wordpress_https_buffer_start');
add_action('send_headers', 'wordpress_https_buffer_start');
add_action('wp_footer', 'wordpress_https_buffer_end');
?>